import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.j256.ormlite.jdbc.JdbcConnectionSource;
import com.j256.ormlite.stmt.PreparedQuery;
import com.j256.ormlite.stmt.QueryBuilder;
import com.j256.ormlite.support.ConnectionSource;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class AccountApp {

    // we are using a MySQl database
    private final static String DATABASE_URL = "jdbc:mysql://localhost:3306/orm_lab?useUnicode=true&useJDBCCompliantTimezoneShift=true&useLegacyDatetimeCode=false&serverTimezone=UTC";
    private final static String YOURUSERNAME = "root";
    private final static String YOURPASSWORD = null;

    private Dao<Account, Integer> accountDao;
    private AccountService accountService;
    private Account activeAccount;
    private boolean running = true;
    private Console console = new Console();

    public static void main(String[] args) throws Exception {
        // turn our static method into an instance of Main
        new AccountApp().doMain(args);
    }

    private void doMain(String[] args) throws Exception {
        ConnectionSource connectionSource = null;
        try {
            connectionSource = new JdbcConnectionSource(DATABASE_URL, YOURUSERNAME, YOURPASSWORD);
            setupDao(connectionSource);
            setupAccountServ(accountDao);
            run();
            System.out.println("\n\nStarting...\n\n");
        } finally {
            // destroy the data source which should close underlying connections
            if (connectionSource != null) {
                connectionSource.close();
            }
        }
    }

    private void run() throws Exception{
        Account account = createAccount();
        accountService.create(account);
        boolean success = signin();
        if(success){
            activeAccount = account;
            while(running == true) {
                menu();
            }
        }
    }

    public int changeUsername(){
        Console.println("Please enter your new Username: ");
        String user = console.next();
        return accountService.changeUsername(activeAccount, user);
    }

    public void deleteAccount(){
        boolean verify = verifyPassword();
        if(verify) {
            accountService.deleteAccount(activeAccount);
        } else {
            Console.println("The password you've entered is incorrect");
        }
    }

    public int changePassword(){
        Console.println("Please enter your new Password: ");
        String password = console.next();
        return accountService.changePassword(activeAccount, password);
    }

    private void menu(){
        Console.printMenue();
        String command = "";
        while(!command.equals("quit")) {
            command = console.next();
            switch (command.toLowerCase()) {
                case "change password":
                    changePassword();
                    break;
                case "change username":
                    changeUsername();
                    break;
                case "delete account":
                    deleteAccount();
                    Console.println("You have been signed out automatically.");
                    running = false;
                    break;
                case "quit":
                    running = false;
                    break;
                default:
                    Console.println("Sorry, that command is not recognized");
            }
        }
    }

    public Account createAccount() throws Exception {
        List<Account> result;
        Console.println("Hello, to sign up, please enter a USERNAME:");
        String input, username = "";
        while(username.equals("")) {
            input = console.next();
            result = this.accountService.querey(input, Account.NAME_FIELD_NAME);
            if (result.size() > 0) {
                Console.println("That username is already taken. Please try again: ");
            } else {
                username = input;
            }
        }
        Console.println("Welcome " + username + ", please create a PASSWORD:");
        String password = console.next();
        Account account = new Account(username, password);
        Console.println("Your account has been created!");
        return account;
    }

    public boolean verifyPassword(){
        Console.println("Please enter your password:");
        String password = console.next();
        return accountService.verifyPassword(activeAccount, password);
    }

    public String getUsername(){
        List<Account> result;
        Console.println("Hello, to sign-in please enter your USERNAME:");
        String nameSubmition = "", input = "";
        while(nameSubmition == "") {
            input = console.next();
            result = this.accountService.querey(input, Account.NAME_FIELD_NAME);
            if (result.size() == 0) {
                Console.println("This user specified does not exist.");
            } else {
                return input;
            }
        }
        return null;
    }

    public boolean signinPassword(String username){
        Console.println("Please enter your PASSWORD:");
        List<Account> account = new ArrayList<>();
        while(account.size() == 0) {
            String passSubmition = console.next();
            account = this.accountService.multipleCaseQuery(username, passSubmition, Account.NAME_FIELD_NAME, Account.PASSWORD_FIELD_NAME);
            if (account.size() == 1) {
                Console.println("You have successfully signed in");
                return true;
            } else {
                Console.println("Incorrect password, please try again: ");
            }
        }
        return false;
    }

    public boolean signin() throws Exception{
        String username = getUsername();
        return signinPassword(username);
    }
    /**
     * Setup our  DAOs
     */
    private void setupDao(ConnectionSource connectionSource) throws Exception {
        accountDao = DaoManager.createDao(connectionSource, Account.class);
    }

    private void setupAccountServ(Dao<Account, Integer> dao){
        this.accountService = new AccountService(dao);
    }
}